#!/usr/bin/python

import Tkinter
import tkMessageBox
import Queue
import socket
import time
import threading

class GUI(Tkinter.Frame):
	def __init__(self, master=None):
		Tkinter.Frame.__init__(self, master)
		self.master = master
		self.create_widgets()
		self.grid()

		self.recv_queue = Queue.Queue()
		self.send_queue = None
		self.my_nick = ""

		self._blink_toggle = False
		self._want_blink = False

		self.after(100, self.check_queue)
		self.after(500, self.blink_title)

	def blink_title(self):
		if self._want_blink:
			if self.focus_get() == None:
				if self._blink_toggle:
					self._blink_toggle = False
					self.master.title("NEW MESSAGE")
				else:
					self._blink_toggle = True
					self.master.title("-")
			else:
				self.master.title("pimc")
				self._want_blink = False
		self.after(500, self.blink_title)

	def check_queue(self):
		try:
			while True:
				type, data = self.recv_queue.get_nowait()
				if type == "quit":
					tkMessageBox.showerror("Connection Lost", \
						"Connection Lost")
					self.quit()
				elif type == "msg":
					try:
						name = data[:data.index(':')]
						if name == '@':
							color = "serv"
						elif name == self.my_nick:
							color = "self"
						else:
							color = "them"
					except ValueError:
						color = "none"
					self.conversation.config(state=Tkinter.NORMAL)
					self.conversation.insert(Tkinter.END, data, (color))
					self.conversation.config(state=Tkinter.DISABLED)
					self.conversation.yview_moveto(1.0)
					self._want_blink = True
		except Queue.Empty:
			pass
		self.after(100, self.check_queue)

	def send_message(self, event=None):
		data = self.message.get()
		if len(data) == 0:
			return
		self.message.delete(0, Tkinter.END)
		self.send_queue.put(data)

	def create_widgets(self):
		self.conversation = Tkinter.Text(self)
		self.scrollbar    = Tkinter.Scrollbar(self)
		self.message      = Tkinter.Entry(self)
		self.send         = Tkinter.Button(self)

		self.scrollbar.config(command=self.conversation.yview)
		self.conversation.config(yscrollcommand=self.scrollbar.set)
		self.conversation.config(state=Tkinter.DISABLED)

		self.conversation.tag_config("none", foreground="black")
		self.conversation.tag_config("self", foreground="red")
		self.conversation.tag_config("them", foreground="blue")
		self.conversation.tag_config("serv", foreground="green")

		self.send["text"] = "Send"
		self.send["command"] = self.send_message

		self.message.bind("<Return>", self.send_message)

		self.conversation.grid(row=0, column=0, columnspan=2)
		self.scrollbar.grid(   row=0, column=2, sticky="n,s")
		self.message.grid(     row=1, column=0, sticky="w,e")
		self.send.grid(        row=1, column=1, columnspan=2, sticky="e")

		self.message.focus_set()

class Connection(object):
	def __init__(self, server_host, server_port):
		self.recv_queue = Queue.Queue()
		self.send_queue = None

		self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self._sock.connect((server_host, server_port))
		self._sock.setblocking(0)

		self._thread = threading.Thread(target=self.loop)

	def loop(self):
		while not self._abort:
			try:
				try:
					while True:
						data = self.recv_queue.get_nowait()
						self._sock.send(data.encode('utf-8'))
				except Queue.Empty:
					pass
				data = self._sock.recv(4096)
				if len(data) > 0:
					try:
						self.send_queue.put(('msg', data.decode('utf-8')))
					except UnicodeDecodeError:
						self.send_queue.put(('msg', \
							'@: Received data that was not UTF-8 encoded.'))
			except socket.error, e:
				# 11 = EAGAIN on Linux, 10035 = Windows.
				if e[0] != 11 and e[0] != 10035:
					self._sock.close()
					self.send_queue.put(('quit', None))
					return
			time.sleep(0.01)

	def start(self):
		self._abort = False
		self._thread.start()

	def stop(self):
		self._abort = True
		self._thread.join()

if __name__ == "__main__":
	import sys

	if len(sys.argv) != 4:
		print "Usage: %s <host> <port> <username>" % (sys.argv[0])
		sys.exit(1)

	root = Tkinter.Tk()
	root.title("pimc")
	root.resizable(0,0)

	gui = GUI(master=root)

	try:
		con = Connection(sys.argv[1], int(sys.argv[2]))
	except:
		tkMessageBox.showerror("Connection Failed", \
			"Unable to establish connection to %s" % (sys.argv[1]))
		sys.exit(1)

	# Automatic login:
	con.recv_queue.put("/nick %s" % (sys.argv[3]))
	gui.my_nick = sys.argv[3]

	# Link queues together:
	con.send_queue = gui.recv_queue
	gui.send_queue = con.recv_queue

	con.start()
	gui.mainloop()
	con.stop()

